## Functions

<dl>
<dt><a href="#create_j_edges">create_j_edges(_nodes, [domLevels], [domTree], [idoms])</a> ⇒ <code>*</code></dt>
<dd><p>This will create and return the J-edges of a graph. The J-edges, or Join edges,
make up one-half of the DJ-graph. For more information read the documentation
for the DJ-graph.</p>
<p>You need only pass the nodes of the graph to this function. The rest of the parameters
are optional and will be computed if not provided. I allow the options to pass them
in case you already have them calculated from elsewhere, just to make things a bit faster.
If no arguments are provided other than the basic vertices, it will compute the immediate
dominators, create the dominator tree, and compute the levels, and discard all of those results.
Not a big deal unless you&#39;re dealing with very large graphs, in which case you should
calculate those separately and provide them as inputs here.</p>
</dd>
<dt><a href="#create_levels">create_levels(nodes)</a> ⇒ <code>Array.&lt;number&gt;</code></dt>
<dd><p>Calculate the level of each node in terms of how many edges it takes to reach
the root. For the sake of simplicity, this uses a BFS to compute depth values.</p>
</dd>
<dt><a href="#create_nodes">create_nodes(_nodes, [idoms], idoms)</a></dt>
<dd><p>A convenience method. It returns an array of object, one for each nodes in the graph,
and in that order, that holds most of the information you could want for working
with graphs.</p>
<p>Specifically, each node looks as descibed in the typedef for GraphNode.</p>
</dd>
<dt><a href="#create_dj_graph">create_dj_graph(nodes, idoms, domTree)</a></dt>
<dd><p>Returns a DJ-graph which is a graph that consts of the dominator tree and select
join edges from the input graph.</p>
</dd>
<dt><a href="#make_dom">make_dom(opts)</a></dt>
<dd></dd>
<dt><a href="#iterative">iterative(succs, [startIndex], [flat])</a> ⇒ <code>Array.&lt;number&gt;</code></dt>
<dd></dd>
<dt><a href="#check">check(vertices, idoms)</a></dt>
<dd><p>Find dominance frontiers</p>
</dd>
<dt><a href="#frontiers_from_preds">frontiers_from_preds(preds, idoms)</a></dt>
<dd><p>Find dominance frontiers</p>
</dd>
<dt><a href="#frontiers_from_succs">frontiers_from_succs(succs, idoms)</a></dt>
<dd><p>Find dominance frontiers</p>
</dd>
<dt><a href="#lt">lt(nodes, [startIndex], [flat])</a></dt>
<dd></dd>
<dt><a href="#condRefToSelf">condRefToSelf(seed, [chk], [dest])</a> ⇒ <code>Array.&lt;Array.&lt;number&gt;&gt;</code></dt>
<dd></dd>
<dt><a href="#simpleRefToSelf">simpleRefToSelf(seed, [dest])</a> ⇒ <code>Array.&lt;Array.&lt;number&gt;&gt;</code></dt>
<dd></dd>
</dl>

## Typedefs

<dl>
<dt><a href="#GraphNode">GraphNode</a> : <code>object</code></dt>
<dd></dd>
<dt><a href="#DomWalkerOptions">DomWalkerOptions</a> : <code>object</code></dt>
<dd></dd>
</dl>

<a name="create_j_edges"></a>

## create_j_edges(_nodes, [domLevels], [domTree], [idoms]) ⇒ <code>\*</code>
This will create and return the J-edges of a graph. The J-edges, or Join edges,
make up one-half of the DJ-graph. For more information read the documentation
for the DJ-graph.

You need only pass the nodes of the graph to this function. The rest of the parameters
are optional and will be computed if not provided. I allow the options to pass them
in case you already have them calculated from elsewhere, just to make things a bit faster.
If no arguments are provided other than the basic vertices, it will compute the immediate
dominators, create the dominator tree, and compute the levels, and discard all of those results.
Not a big deal unless you're dealing with very large graphs, in which case you should
calculate those separately and provide them as inputs here.

**Kind**: global function  
**See**: create_dj_graph  

| Param | Type | Description |
| --- | --- | --- |
| _nodes | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | An array of arrays of successors indices, as always |
| [domLevels] | <code>Array.&lt;number&gt;</code> | The levels (or depth) of the nodes in the dominator tree |
| [domTree] | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | The dominator tree in the standard format, same as _nodes |
| [idoms] | <code>Array.&lt;number&gt;</code> | The immediate dominators |

<a name="create_levels"></a>

## create_levels(nodes) ⇒ <code>Array.&lt;number&gt;</code>
Calculate the level of each node in terms of how many edges it takes to reach
the root. For the sake of simplicity, this uses a BFS to compute depth values.

**Kind**: global function  

| Param | Type |
| --- | --- |
| nodes | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 

<a name="create_nodes"></a>

## create_nodes(_nodes, [idoms], idoms)
A convenience method. It returns an array of object, one for each nodes in the graph,
and in that order, that holds most of the information you could want for working
with graphs.

Specifically, each node looks as descibed in the typedef for GraphNode.

**Kind**: global function  
**See**: GraphNode  

| Param | Type | Description |
| --- | --- | --- |
| _nodes | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | The usual graph nodes |
| [idoms] | <code>Array.&lt;number&gt;</code> | The immediate dominators, if not provided, they will be computed |
| idoms |  |  |

<a name="create_dj_graph"></a>

## create_dj_graph(nodes, idoms, domTree)
Returns a DJ-graph which is a graph that consts of the dominator tree and select
join edges from the input graph.

**Kind**: global function  

| Param |
| --- |
| nodes | 
| idoms | 
| domTree | 

<a name="make_dom"></a>

## make_dom(opts)
**Kind**: global function  

| Param | Type |
| --- | --- |
| opts | [<code>DomWalkerOptions</code>](#DomWalkerOptions) \| <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 


* [make_dom(opts)](#make_dom)
    * [~iterated_dominance_frontier(fn, [defs])](#make_dom..iterated_dominance_frontier) ⇒ <code>Set.&lt;number&gt;</code>
    * [~forStrictDominators(fn, dom)](#make_dom..forStrictDominators)
    * [~forDominators(fn, dom)](#make_dom..forDominators)
    * [~strictDominators(dom)](#make_dom..strictDominators) ⇒ <code>Array.&lt;number&gt;</code>
    * [~dominators()](#make_dom..dominators) ⇒ <code>Array.&lt;number&gt;</code>
    * [~strictlyDominates(from, [to])](#make_dom..strictlyDominates) ⇒ <code>boolean</code> \| <code>Array.&lt;number&gt;</code>
    * [~dominates(from, [to])](#make_dom..dominates) ⇒ <code>boolean</code> \| <code>Array.&lt;number&gt;</code>
    * [~forStrictlyDominates(fn, from, [notStrict])](#make_dom..forStrictlyDominates)
    * [~forDominates(fn, from)](#make_dom..forDominates)
    * [~forDominanceFrontier(fn, from)](#make_dom..forDominanceFrontier)
    * [~dominanceFrontier(from)](#make_dom..dominanceFrontier) ⇒ <code>Array.&lt;number&gt;</code>
    * [~forIteratedDominanceFrontier(fn, from)](#make_dom..forIteratedDominanceFrontier)
    * [~forPrunedIteratedDominanceFrontier(fn, from)](#make_dom..forPrunedIteratedDominanceFrontier)
    * [~iteratedDominanceFrontier(from)](#make_dom..iteratedDominanceFrontier) ⇒ <code>Array.&lt;number&gt;</code>
    * [~_forDominanceFrontier(fn, from)](#make_dom.._forDominanceFrontier)

<a name="make_dom..iterated_dominance_frontier"></a>

### make_dom~iterated_dominance_frontier(fn, [defs]) ⇒ <code>Set.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> \| <code>Array.&lt;number&gt;</code> | 
| [defs] | <code>Array.&lt;number&gt;</code> \| <code>function</code> | 

<a name="make_dom..forStrictDominators"></a>

### make_dom~forStrictDominators(fn, dom)
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| dom | <code>number</code> | 

<a name="make_dom..forDominators"></a>

### make_dom~forDominators(fn, dom)
Note: This will visit the dominators starting with the 'to' node and moving up the idom tree
until it gets to the root.

**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| dom | <code>number</code> | 

<a name="make_dom..strictDominators"></a>

### make_dom~strictDominators(dom) ⇒ <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| dom | <code>number</code> | 

<a name="make_dom..dominators"></a>

### make_dom~dominators() ⇒ <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  
<a name="make_dom..strictlyDominates"></a>

### make_dom~strictlyDominates(from, [to]) ⇒ <code>boolean</code> \| <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| from | <code>number</code> | 
| [to] | <code>number</code> | 

<a name="make_dom..dominates"></a>

### make_dom~dominates(from, [to]) ⇒ <code>boolean</code> \| <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| from | <code>number</code> | 
| [to] | <code>number</code> | 

<a name="make_dom..forStrictlyDominates"></a>

### make_dom~forStrictlyDominates(fn, from, [notStrict])
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type | Default |
| --- | --- | --- |
| fn | <code>function</code> |  | 
| from | <code>number</code> |  | 
| [notStrict] | <code>boolean</code> | <code>false</code> | 

<a name="make_dom..forDominates"></a>

### make_dom~forDominates(fn, from)
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| from | <code>number</code> | 

<a name="make_dom..forDominanceFrontier"></a>

### make_dom~forDominanceFrontier(fn, from)
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| from | <code>number</code> | 

<a name="make_dom..dominanceFrontier"></a>

### make_dom~dominanceFrontier(from) ⇒ <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| from | <code>number</code> | 

<a name="make_dom..forIteratedDominanceFrontier"></a>

### make_dom~forIteratedDominanceFrontier(fn, from)
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| from | <code>Array.&lt;number&gt;</code> | 

<a name="make_dom..forPrunedIteratedDominanceFrontier"></a>

### make_dom~forPrunedIteratedDominanceFrontier(fn, from)
This is a close relative of forIteratedDominanceFrontier(), which allows the
given functor to return false to indicate that we don't wish to consider the given block.
Useful for computing pruned SSA form.

**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| from | <code>Array.&lt;number&gt;</code> | 

<a name="make_dom..iteratedDominanceFrontier"></a>

### make_dom~iteratedDominanceFrontier(from) ⇒ <code>Array.&lt;number&gt;</code>
**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| from | <code>Array.&lt;number&gt;</code> | 

<a name="make_dom.._forDominanceFrontier"></a>

### make_dom~_forDominanceFrontier(fn, from)
Paraphrasing from http:*en.wikipedia.org/wiki/Dominator_(graph_theory):

>    "The dominance frontier of a block 'from' is the set of all blocks 'to' such that
>    'from' dominates an immediate predecessor of 'to', but 'from' does not strictly
>    dominate 'to'."

A useful corner case to remember: a block may be in its own dominance frontier if it has
a loop edge to itself, since it dominates itself and so it dominates its own immediate
predecessor, and a block never strictly dominates itself.

**Kind**: inner method of [<code>make_dom</code>](#make_dom)  

| Param | Type |
| --- | --- |
| fn | <code>function</code> | 
| from | <code>number</code> | 

<a name="iterative"></a>

## iterative(succs, [startIndex], [flat]) ⇒ <code>Array.&lt;number&gt;</code>
**Kind**: global function  

| Param | Type | Default |
| --- | --- | --- |
| succs | <code>Array.&lt;(Array.&lt;number&gt;\|number)&gt;</code> |  | 
| [startIndex] | <code>number</code> | <code>0</code> | 
| [flat] | <code>boolean</code> | <code>true</code> | 

<a name="iterative..find_idoms"></a>

### iterative~find_idoms(index)
**Kind**: inner method of [<code>iterative</code>](#iterative)  

| Param | Type |
| --- | --- |
| index | <code>number</code> | 

<a name="check"></a>

## check(vertices, idoms)
Find dominance frontiers

**Kind**: global function  

| Param | Type |
| --- | --- |
| vertices | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| idoms | <code>Array.&lt;?number&gt;</code> | 

<a name="frontiers_from_preds"></a>

## frontiers_from_preds(preds, idoms)
Find dominance frontiers

**Kind**: global function  

| Param | Type |
| --- | --- |
| preds | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| idoms | <code>Array.&lt;?number&gt;</code> | 

<a name="frontiers_from_succs"></a>

## frontiers_from_succs(succs, idoms)
Find dominance frontiers

**Kind**: global function  

| Param | Type |
| --- | --- |
| succs | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| idoms | <code>Array.&lt;?number&gt;</code> | 

<a name="lt"></a>

## lt(nodes, [startIndex], [flat])
**Kind**: global function  

| Param | Type | Default |
| --- | --- | --- |
| nodes | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> |  | 
| [startIndex] | <code>number</code> | <code>0</code> | 
| [flat] | <code>boolean</code> | <code>true</code> | 

<a name="condRefToSelf"></a>

## condRefToSelf(seed, [chk], [dest]) ⇒ <code>Array.&lt;Array.&lt;number&gt;&gt;</code>
**Kind**: global function  
**Returns**: <code>Array.&lt;Array.&lt;number&gt;&gt;</code> - }  

| Param | Type |
| --- | --- |
| seed | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| [chk] | <code>function</code> | 
| [dest] | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 

<a name="simpleRefToSelf"></a>

## simpleRefToSelf(seed, [dest]) ⇒ <code>Array.&lt;Array.&lt;number&gt;&gt;</code>
**Kind**: global function  
**Returns**: <code>Array.&lt;Array.&lt;number&gt;&gt;</code> - }  

| Param | Type |
| --- | --- |
| seed | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| [dest] | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 

<a name="GraphNode"></a>

## GraphNode : <code>object</code>
**Kind**: global typedef  
**Properties**

| Name | Type | Description |
| --- | --- | --- |
| id | <code>number</code> | The index of this node in the original array |
| succs | <code>Array.&lt;number&gt;</code> | The successor node indices |
| preds | <code>Array.&lt;number&gt;</code> | The predecessor node indices |
| domSuccs | <code>Array.&lt;number&gt;</code> | The dominator tree successor indices |
| idom | <code>number</code> | The immediate dominator and, of course, dominator tree predecessor |
| level | <code>number</code> | The depth (or level) of the vertex |
| domLevel | <code>number</code> | The depth in the dominator tree |
| jSuccs | <code>Array.&lt;number&gt;</code> | The successor J-edges, if any, of this node |
| jPreds | <code>Array.&lt;number&gt;</code> | The predecessor J-edges, if any, of this node |

<a name="DomWalkerOptions"></a>

## DomWalkerOptions : <code>object</code>
**Kind**: global typedef  
**Properties**

| Name | Type |
| --- | --- |
| nodes | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| idoms | <code>Array.&lt;?number&gt;</code> | 
| domTree | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| jEdges | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| frontiers | <code>Array.&lt;Array.&lt;number&gt;&gt;</code> | 
| djGraph | <code>Array.&lt;Array.&lt;Array.&lt;number&gt;&gt;&gt;</code> | 
| domLevels | <code>Array.&lt;number&gt;</code> | 

