# dom-tree.js API documentation

<!-- div class="toc-container" -->

<!-- div -->

## `Methods`
* <a href="#create_dj_graphnodes-idoms-domtree">`create_dj_graph`</a>
* <a href="#create_j_edges_nodes-domlevels-domtree-idoms">`create_j_edges`</a>
* <a href="#create_levelsnodes">`create_levels`</a>
* <a href="#create_nodes_nodes-idoms">`create_nodes`</a>
* <a href="#dominancefrontierfrom">`dominanceFrontier`</a>
* <a href="#dominatesfrom-to">`dominates`</a>
* <a href="#dominators">`dominators`</a>
* <a href="#fordominatesfn-from">`forDominates`</a>
* <a href="#fordominatorsfn-dom">`forDominators`</a>
* <a href="#foriterateddominancefrontierfn-from">`forIteratedDominanceFrontier`</a>
* <a href="#forprunediterateddominancefrontierfn-from">`forPrunedIteratedDominanceFrontier`</a>
* <a href="#forstrictlydominatesfn-from-notstrict">`forStrictlyDominates`</a>
* <a href="#iterateddominancefrontierfrom">`iteratedDominanceFrontier`</a>
* <a href="#iterated_dominance_frontierfn-defs">`iterated_dominance_frontier`</a>
* <a href="#strictdominatorsdom">`strictDominators`</a>

<!-- /div -->

<!-- /div -->

<!-- div class="doc-container" -->

<!-- div -->

## `Methods`

<!-- div -->

<h3 id="create_dj_graphnodes-idoms-domtree"><code>create_dj_graph(nodes, [idoms], [domTree])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L205 "View in source") [&#x24C9;][1]

Returns a DJ-graph which is a graph that consts of the dominator tree and select
join edges from the input graph.

#### Arguments
1. `nodes` *(number&#91;&#93;&#91;&#93;)*: Graph in the usual format
2. `[idoms]` *(number&#91;&#93;)*: Immediate dominators, if omiteed, they will be computed
3. `[domTree]` *(number&#91;&#93;&#91;&#93;)*: Dominator tree, it omitted, will be computed

---

<!-- /div -->

<!-- div -->

<h3 id="create_j_edges_nodes-domlevels-domtree-idoms"><code>create_j_edges(_nodes, [domLevels], [domTree], [idoms])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L87 "View in source") [&#x24C9;][1]

This will create and return the J-edges of a graph. The J-edges, or Join edges,
make up one-half of the DJ-graph. For more information read the documentation
for the DJ-graph.
<br>
<br>
You need only pass the nodes of the graph to this function. The rest of the parameters
are optional and will be computed if not provided. I allow the options to pass them
in case you already have them calculated from elsewhere, just to make things a bit faster.
If no arguments are provided other than the basic vertices, it will compute the immediate
dominators, create the dominator tree, and compute the levels, and discard all of those results.
Not a big deal unless you're dealing with very large graphs, in which case you should
calculate those separately and provide them as inputs here.

#### Arguments
1. `_nodes` *(number&#91;&#93;&#91;&#93;)*: An array of arrays of successors indices, as always
2. `[domLevels]` *(number&#91;&#93;): The levels &#42;(or depth)*&#42; of the nodes in the dominator tree
3. `[domTree]` *(number&#91;&#93;&#91;&#93;)*: The dominator tree in the standard format, same as _nodes
4. `[idoms]` *(number&#91;&#93;)*: The immediate dominators

---

<!-- /div -->

<!-- div -->

<h3 id="create_levelsnodes"><code>create_levels(nodes)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L126 "View in source") [&#x24C9;][1]

Calculate the level of each node in terms of how many edges it takes to reach
the root. For the sake of simplicity, this uses a BFS to compute depth values.

#### Arguments
1. `nodes` *(number&#91;&#93;&#91;&#93;)*: The graph

---

<!-- /div -->

<!-- div -->

<h3 id="create_nodes_nodes-idoms"><code>create_nodes(_nodes, [idoms])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L165 "View in source") [&#x24C9;][1]

A convenience method. It returns an array of object, one for each nodes in the graph,
and in that order, that holds most of the information you could want for working
with graphs.
<br>
<br>
Specifically, each node looks as descibed in the typedef for GraphNode.

#### Arguments
1. `_nodes` *(number&#91;&#93;&#91;&#93;)*: The usual graph nodes
2. `[idoms]` *(number&#91;&#93;)*: The immediate dominators, if not provided, they will be computed

---

<!-- /div -->

<!-- div -->

<h3 id="dominancefrontierfrom"><code>dominanceFrontier(from)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L500 "View in source") [&#x24C9;][1]



#### Arguments
1. `from` *(number)*:

---

<!-- /div -->

<!-- div -->

<h3 id="dominatesfrom-to"><code>dominates(from, [to])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L441 "View in source") [&#x24C9;][1]



#### Arguments
1. `from` *(number)*:
2. `[to]` *(number)*:

---

<!-- /div -->

<!-- div -->

<h3 id="dominators"><code>dominators()</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L405 "View in source") [&#x24C9;][1]



---

<!-- /div -->

<!-- div -->

<h3 id="fordominatesfn-from"><code>forDominates(fn, from)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L473 "View in source") [&#x24C9;][1]



#### Arguments
1. `fn` *(function)*:
2. `from` *(number)*:

---

<!-- /div -->

<!-- div -->

<h3 id="fordominatorsfn-dom"><code>forDominators(fn, dom)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L384 "View in source") [&#x24C9;][1]

Note: This will visit the dominators starting with the 'to' node and moving up the idom tree
until it gets to the root.

#### Arguments
1. `fn` ():
2. `dom` *(number)*:

---

<!-- /div -->

<!-- div -->

<h3 id="foriterateddominancefrontierfn-from"><code>forIteratedDominanceFrontier(fn, from)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L512 "View in source") [&#x24C9;][1]



#### Arguments
1. `fn` *(function)*:
2. `from` *(number&#91;&#93;)*:

---

<!-- /div -->

<!-- div -->

<h3 id="forprunediterateddominancefrontierfn-from"><code>forPrunedIteratedDominanceFrontier(fn, from)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L530 "View in source") [&#x24C9;][1]

This is a close relative of forIteratedDominanceFrontier(), which allows the
given functor to return false to indicate that we don't wish to consider the given block.
Useful for computing pruned SSA form.

#### Arguments
1. `fn` *(function)*:
2. `from` *(number&#91;&#93;)*:

---

<!-- /div -->

<!-- div -->

<h3 id="forstrictlydominatesfn-from-notstrict"><code>forStrictlyDominates(fn, from, [notStrict])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L457 "View in source") [&#x24C9;][1]



#### Arguments
1. `fn` *(function)*:
2. `from` *(number)*:
3. `[notStrict]` *(boolean)*: =false&#93;

---

<!-- /div -->

<!-- div -->

<h3 id="iterateddominancefrontierfrom"><code>iteratedDominanceFrontier(from)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L543 "View in source") [&#x24C9;][1]



#### Arguments
1. `from` *(number&#91;&#93;)*:

---

<!-- /div -->

<!-- div -->

<h3 id="iterated_dominance_frontierfn-defs"><code>iterated_dominance_frontier(fn, [defs])</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L274 "View in source") [&#x24C9;][1]



#### Arguments
1. `fn` *(function|number&#91;&#93;)*:
2. `[defs]` *(function|number&#91;&#93;)*:

---

<!-- /div -->

<!-- div -->

<h3 id="strictdominatorsdom"><code>strictDominators(dom)</code></h3>
[&#x24C8;](https://github.com/julianjensen/dominators/src/dom-tree.js#L393 "View in source") [&#x24C9;][1]



#### Arguments
1. `dom` *(number)*:

---

<!-- /div -->

<!-- /div -->

<!-- /div -->

 [1]: #methods "Jump back to the TOC."
